#!/usr/bin/env python
# -*- coding: utf-8 -*-
from random import randint
from .agents import agents

import requests
import re
import os
import socket
import time
import urllib2
import urllib
import json
# Download SocksiPy - A Python SOCKS client module. ( http://code.google.com/p/socksipy-branch/downloads/list )
import socks

ENV = os.environ.get('ENV', 'production')
# On wich port Tor's proxy is avalaible
TOR_PROXY_PORT = int( os.environ.get('TOR_PROXY_PORT', 9050) )
# Maximum throttle delay
THROTTLE_MAX = int( os.environ.get('THROTTLE_MAX', 3) )

def setup():
    # Equal 0 if the TOR_PROXY_PORT is open
    if socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect_ex(('127.0.0.1', TOR_PROXY_PORT)) == 0:
        # Tor must listen for SOCKS request on 9050
        socks.setdefaultproxy(socks.PROXY_TYPE_SOCKS5, "127.0.0.1", TOR_PROXY_PORT)
        socket.socket = socks.socksocket
        # Notify the user
        print 'Your requests are routed throught Tor'
    # Ensure the timeout is limited to 20 seconds
    socket.setdefaulttimeout(20)

# fetch_page sets up the scraper and the VPN, returns the contents of the page

def fetch_page(url, throttle=(ENV == 'production'), postdata = None):

    if throttle: time.sleep(randint(0, THROTTLE_MAX))

    request = urllib2.Request(url, postdata)
    request.add_header('Cache-Control','max-age=0')
    request.add_header('User-Agent', agents[randint(0,len(agents)-1)], )
    request.add_header('Accept', 'application/json,text/plain,text/html,application/xhtml+xml,application/xml, */*')
    response = urllib2.urlopen(request)

    return response.read()


def regex(needle, haystk, transform=lambda a:a):
    haystk = str(haystk) if type(haystk) is not str else haystk
    match = re.search(r''+needle+'', haystk, re.MULTILINE|re.DOTALL|re.IGNORECASE)
    res = match.group(1) if match != None else None
    return transform(res)

def booleanize(val, null=None):
    if type(val) is bool:
        return val
    elif type(val) is int:
        return bool(val)
    elif val == 'n':
        return False
    elif val == 'y':
        return True
    else:
        return null

def convert(amount, from_currency):

    currencies = {
        "DKK": 0.13,
        "CZK": 0.037,
        "PLN": 0.24,
        "GBP": 1.40,
        "SEK": 0.11,
        "CHF": 0.920587,
        "HUF": 0.0032,
        "BRL": 0.25,
        "VND": 0.000039,
        "EUR": 1
    }

    try:
        amount = float(amount)
    except (ValueError, TypeError):
        print "Invalid amount"
        return False

    return amount * currencies[from_currency]

def clean_amounts(totalRent=0, baseRent=0, serviceCharge=0, livingSpace=None, currency='EUR'):

    baseRent = float(baseRent)
    totalRent = float(totalRent)
    livingSpace = float(livingSpace)

    try:
        serviceCharge = float(serviceCharge)
    except (ValueError, TypeError):
        serviceCharge = 0

    # Computes totalRent and baseRent if they're missing
    if totalRent == 0: totalRent = baseRent + serviceCharge
    if baseRent == 0: baseRent = totalRent - serviceCharge

    # Checks for bogus ads
    if totalRent < 30 or totalRent > 25000 or livingSpace < 3 or livingSpace > 1000:
        return False

    if serviceCharge == 0:
        serviceCharge = None

    return {
        "baseRent": convert(baseRent, currency),
        "baseRentOriginalCurrency": baseRent,
        "serviceCharge": convert(serviceCharge, currency),
        "serviceChargeOriginalCurrency": serviceCharge,
        "totalRent": convert(totalRent, currency),
        "totalRentOriginalCurrency": totalRent,
        "livingSpace": livingSpace,
        "pricePerSqm": convert(baseRent, currency) / livingSpace,
        "currency": currency
    }

def geocode(address, country):

    geocoder  = "http://nominatim.openstreetmap.org/search?"
    geocoder += "format=json&"
    geocoder += "limit=10&"
    geocoder += "bounded=1&"
    geocoder += "osm_type=N&"
    geocoder += "countrycodes=%s&" % country
    geocoder += "q=%s&" % urllib.quote_plus(address)

    page = fetch_page(geocoder)
    data = json.loads(page)

    if len(data) > 0:
        return {
            "lat" : float(data[0]["lat"]),
            "lng" : float(data[0]["lon"])
        }
    else:
        return {
            "lat" : None,
            "lng" : None
        }

def convert_in_meters(string, separator):
    if (separator in string):
        feet = string.split(separator)[0]
        inches = string.split(separator)[1]
        
        inches = inches.replace(" ", "")
        if inches != "" and inches is not None:
            inches = float(inches) * 0.025
        else:
            inches = 0
        return float(feet) * 0.305 + inches

    else:
        feet = string
        return float(feet)

def convert_in_sqm(string):
    return float(string) * 0.092903
    
def find_living_space(desc, no_rooms):
    patterns = [
        {
            # Looks for rooms in feet under the format 5'10" x 3'1"
            "format_finder_expression": "\d{1,2}'[ ]*\d{1,2}\" x \d{1,2}'[ ]*\d{1,2}\"",
            "expression": "(\d{1,2}'[ ]*\d{0,2})(?:\")*(?: max)* x (\d{1,2}'[ ]*\d{0,2})(?:\")*",
            "expect" : "several" # several rooms to add up
        },
        {
            # Looks for rooms in feet under the format 10'2 x 9'3
            "format_finder_expression": "\d{1,2}'\d{1,2} x \d{1,2}'\d{1,2}(?!\")",
            "expression": "(\d{1,2}'\d{0,2})(?: max)* x (\d{1,2}'\d{0,2})",
            "expect" : "several"
        },
        {
            # Looks for "646 sq. Ft."
            "format_finder_expression": "sq\. ft\.",
            "expression": "(\d{3,4})[ ]*sq\. ft\.",
            "expect": "one"
        }
    ]


    # Removes conversions in meters of the format " (3.81m)"
    expression_to_remove = "( \(\d{1,2}.\d{1,2}m\))"
    q = re.compile(expression_to_remove)
    for match in q.finditer(desc):
        desc = desc.replace(match.group(1), "")

    # lower cases
    desc = desc.lower()

    for pattern in patterns:
        format_finder_expression = pattern["format_finder_expression"]
    
        if (re.search(format_finder_expression, desc) is not None):

            expression = pattern["expression"]
            p = re.compile(expression)

            living_space = 0

            if pattern["expect"] == "one":
                match = re.search(expression, desc)
                living_space = convert_in_sqm(match.group(1))

            elif pattern["expect"] == "several":
                for match in p.finditer(desc):
                    living_space += convert_in_meters(match.group(1), "'") *  convert_in_meters(match.group(2), "'")

            if (no_rooms != "NULL" and no_rooms != "None"):
                if living_space/float(no_rooms) < 12 or living_space/float(no_rooms) > 40:
                    return None
                else:
                    return living_space
            



from .browser import regex as fast_regex

import itertools
import abc
import reporting
import simplejson as simplejson

class Field(object):
    __metaclass__ = abc.ABCMeta
    _counter = itertools.count()

    def __init__(self, required=False, exception=reporting.BogusError):
        self.required  = required
        self.exception = exception
        # Preserve field order
        self._count = Field._counter.next()

    @abc.abstractmethod
    def extract(self, soup, values):
        return None

class RegexField(Field):

    def __init__(self, selector=None, regex=None, transform=None, html=False,
                required=False, exception=reporting.BogusError):
        self.selector  = selector
        self.regex     = regex
        self.transform = transform
        self.html      = html
        # Call parent constructor
        super(RegexField, self).__init__(required, exception)

    def extract(self, soup, values):
        # Parse page HTML using a selector
        if self.selector is not None:
            # Extract the element containing
            element = soup.select_one(self.selector)
            # Element not found
            if element is None: return None
        # No selector provided the soup might be just text
        else: element = soup
        # Should we extract value as html or text?
        value = element if self.html else element.text.encode('utf-8')
        # Should we apply a regex to extract the value
        value = value if self.regex is None else fast_regex(self.regex, value)
        # Strip extracted values
        value = value.strip() if isinstance(value, str) else value
        # Should we transform the value afterward?
        if hasattr(self.transform, '__call__')  and value is not None:
            # Transforms the value
            value = self.transform(value)
        return value

class AttributeField(Field):

    def __init__(self, selector='a', name='name', transform=None,
                required=False, exception=reporting.BogusError):
        self.selector  = selector
        self.name      = name
        self.transform = transform
        # Call parent constructor
        super(AttributeField, self).__init__(required, exception)

    def extract(self, soup, values):
        # Parse page HTML using a selector
        if self.selector is not None:
            # Extract the element containing
            element = soup.select_one(self.selector)
            # Element not found
            if element is None: return None
        # No selector provided the soup might be just text
        else: element = soup
        # Get the value from the attributes of the element
        try:
            value = element[self.name]
        # There is no such key in the element
        except KeyError:
            value = None
        # Should we transform the value afterward?
        if hasattr(self.transform, '__call__')  and value is not None:
            # Transforms the value
            value = self.transform(value)
        return value

class JsonField(Field):

    def __init__(self, selector=None, transform=None,
                required=False, exception=reporting.BogusError):
        self.selector  = selector
        self.transform = transform
        # Call parent constructor
        super(JsonField, self).__init__(required, exception)

    def extract(self, soup, values):
        # Parse JSON
        json_data = simplejson.loads(str(soup).strip())
        # Finds and return the element
        if self.selector is not None:
            # Split the json selectors
            json_selectors = self.selector.split(".")
            # Creates an intermediary variable upon which to iterate
            json_focus = json_data
            json_count = 0
            for json_selector in json_selectors:
                try:
                    json_focus = json_focus[json_selector]
                    json_count += 1
                    # If this is the last JSON selector
                    if json_count == len(json_selectors):
                        value = str(json_focus).encode('utf-8')
                        if value == "":
                            value = None
                except KeyError:
                    # Selector not found
                    value = None
        else:
            return None
        # Strip extracted values
        value = value.strip() if isinstance(value, str) else value
        if hasattr(self.transform, '__call__'):
            # Transforms the value
            try:
                value = self.transform(value)
            except TypeError:
                value = None
        return value

class ComputedField(Field):
    def __init__(self, fn, required=False, exception=reporting.BogusError):
        # The function called to cumputte this field
        self.fn = fn
        # Call parent constructor
        super(ComputedField, self).__init__(required, exception)
    # The extract method just return the result of the computation function
    def extract(self, soup, values): return self.fn(soup, values)



# -*- coding: utf-8 -*-
from exceptions import ValueError

import db

dupCount = 0

class ReportError(Exception):
    def __init__(self, country, site, siteId):
        Exception.__init__(self)
        # Register attributes
        self.country = country
        self.site = site
        self.siteId = siteId

    def save(self):
        return report(self.country, self.site, self.siteId, self.name)

class DuplicateError(ReportError):
    def __init__(self, country, site, siteId):
        self.name = 'duplicate'
        global dupCount
        dupCount = dupCount + 1
        ReportError.__init__(self, country, site, siteId)

    def save(self):
        return None

class BogusError(ReportError):
    def __init__(self, country, site, siteId):
        self.name = 'bogus'
        ReportError.__init__(self, country, site, siteId)

# Wrong type is for business premises, garage etc.
class WrongtypeError(ReportError):
    def __init__(self, country, site, siteId):
        self.name = 'wrong-type'
        ReportError.__init__(self, country, site, siteId)

class RentMissingError(ReportError):
    def __init__(self, country, site, siteId):
        self.name = 'rent-missing'
        ReportError.__init__(self, country, site, siteId)

class SpaceMissingError(ReportError):
    def __init__(self, country, site, siteId):
        self.name = 'space-missing'
        ReportError.__init__(self, country, site, siteId)

class TimeoutError(ReportError):
    def __init__(self, country, site, siteId):
        self.name = 'timeout'
        ReportError.__init__(self, country, site, siteId)

class UnreachableError(ReportError):
    def __init__(self, country, site, siteId):
        self.name = 'unreachable'
        ReportError.__init__(self, country, site, siteId)

class InvalidError(ReportError):
    def __init__(self, country, site, siteId):
        self.name = 'invalid'
        ReportError.__init__(self, country, site, siteId)

# To quickly catch all reportable errors
REPORTABLE_ERRORS = (DuplicateError, BogusError, WrongtypeError,
                    SpaceMissingError, RentMissingError, InvalidError,
                    TimeoutError, UnreachableError,)

def report(country, site, siteId, name):
    # Check the validity of the report name
    if not name in db.REPORT_NAMES:
        raise ValueError("'%s' is not a valid name!" % name)
    # DB saving
    return db.Report(
        country=country,
        site=site,
        siteId=siteId,
        name=name
    )



agents = [
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/600.7.12 (KHTML, like Gecko) Version/8.0.7 Safari/600.7.12",
	"Mozilla/5.0 (Windows NT 6.1; WOW64; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.132 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.3; WOW64; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10.10; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.125 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.107 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.132 Safari/537.36",
	"Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.125 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.132 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_3) AppleWebKit/600.6.3 (KHTML, like Gecko) Version/8.0.6 Safari/600.6.3",
	"Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.125 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1; WOW64; rv:38.0) Gecko/20100101 Firefox/38.0",
	"Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.107 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.107 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.132 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.89 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko",
	"Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.89 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.132 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (iPad; CPU OS 8_4 like Mac OS X) AppleWebKit/600.1.4 (KHTML, like Gecko) Version/8.0 Mobile/12H143 Safari/600.1.4",
	"Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:38.0) Gecko/20100101 Firefox/38.0",
	"Mozilla/5.0 (Windows NT 5.1; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_5) AppleWebKit/600.7.12 (KHTML, like Gecko) Version/7.1.7 Safari/537.85.16",
	"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.125 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (X11; Linux x86_64; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.125 Safari/537.36",
	"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.132 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1; Trident/7.0; rv:11.0) like Gecko",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.132 Safari/537.36",
	"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/43.0.2357.130 Chrome/43.0.2357.130 Safari/537.36",
	"Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_3) AppleWebKit/600.5.17 (KHTML, like Gecko) Version/8.0.5 Safari/600.5.17",
	"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.135 Safari/537.36 Edge/12.10240",
	"Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10.10; rv:38.0) Gecko/20100101 Firefox/38.0",
	"Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.107 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.124 Safari/537.36",
	"Mozilla/5.0 (iPad; CPU OS 8_3 like Mac OS X) AppleWebKit/600.1.4 (KHTML, like Gecko) Version/8.0 Mobile/12F69 Safari/600.1.4",
	"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.125 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.89 Safari/537.36",
	"Mozilla/5.0 (X11; Linux x86_64; rv:38.0) Gecko/20100101 Firefox/38.0",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11) AppleWebKit/601.1.37 (KHTML, like Gecko) Version/9.0 Safari/601.1.37",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.132 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.89 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.107 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_5) AppleWebKit/600.6.3 (KHTML, like Gecko) Version/7.1.6 Safari/537.85.15",
	"Mozilla/5.0 (Windows NT 10.0; WOW64; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_2) AppleWebKit/600.3.18 (KHTML, like Gecko) Version/8.0.3 Safari/600.3.18",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.3; WOW64; rv:38.0) Gecko/20100101 Firefox/38.0",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.89 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.78.2 (KHTML, like Gecko) Version/6.1.6 Safari/537.78.2",
	"Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.125 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.2; WOW64; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.81 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1; WOW64; rv:31.0) Gecko/20100101 Firefox/31.0",
	"Mozilla/5.0 (Windows NT 6.1; rv:38.0) Gecko/20100101 Firefox/38.0",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11) AppleWebKit/601.1.39 (KHTML, like Gecko) Version/9.0 Safari/601.1.39",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.125 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; Touch; rv:11.0) like Gecko",
	"Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (Windows NT 6.1; rv:31.0) Gecko/20100101 Firefox/31.0",
	"Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.59.10 (KHTML, like Gecko) Version/5.1.9 Safari/534.59.10",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.107 Safari/537.36",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.89 Safari/537.36",
	"Mozilla/5.0 (X11; Linux x86_64; rv:31.0) Gecko/20100101 Firefox/31.0 Iceweasel/31.8.0",
	"Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0;  Trident/5.0)",
	"Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0;  Trident/5.0)",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.134 Safari/537.36",
	"Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_2) AppleWebKit/600.4.10 (KHTML, like Gecko) Version/8.0.4 Safari/600.4.10",
	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_5) AppleWebKit/537.78.2 (KHTML, like Gecko) Version/7.0.6 Safari/537.78.2",
	"Mozilla/5.0 (Windows NT 6.0; rv:39.0) Gecko/20100101 Firefox/39.0",
	"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.107 Safari/537.36",
	"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/43.0.2357.81 Chrome/43.0.2357.81 Safari/537.36"
]



from sqlobject import connectionForURI, sqlhub, SQLObject, sqlbuilder, \
                      BoolCol, StringCol, UnicodeCol, FloatCol, \
                      DateTimeCol, DateCol, DatabaseIndex
import os

# Get DATABASE_URL env variable or use sqlite
DATABASE_URL = os.environ.get('DATABASE_URL', "sqlite://%s" % os.path.abspath("dev.db") )
REPORT_NAMES = ('duplicate', 'bogus', 'rent-missing', 'space-missing',
                'timeout', 'wrong-type', 'unreachable', 'invalid')
# Establish the connection
connection = connectionForURI(DATABASE_URL)
connection.dbEncoding="utf8"

def setup(databaseUrl=DATABASE_URL):
    # Establish the connection
    connection = connectionForURI(databaseUrl)
    connection.dbEncoding="utf8"
    # No pooling
    connection._pool = None
    sqlhub.processConnection = connection
    # Creates tables
    Ad.createTable(ifNotExists=True)
    Report.createTable(ifNotExists=True)


class Ad(SQLObject):
    # "listed" if needs more scraping, "scraped" if it's done
    status = StringCol(length=10, default=None)
    # Name of the website
    site = StringCol(length=30, notNull=True)
    # Date the ad was first scraped
    createdAt = DateTimeCol(default=DateTimeCol.now)
    # The unique ID from the site where it's scrapped from
    siteId = StringCol(length=100, notNull=True)
    # Extra costs (heating mostly)
    serviceCharge = FloatCol(default=None)
    serviceChargeOriginalCurrency = FloatCol(default=None)
    # Base costs (without heating)
    baseRent = FloatCol(default=None)
    baseRentOriginalCurrency = FloatCol(default=None)
    # Total cost
    totalRent = FloatCol(default=None)
    totalRentOriginalCurrency = FloatCol(default=None)
    # Country, 2 letter code
    country = StringCol(length=2, notNull=True)
    # Currency, 3 letter code
    currency = StringCol(length=3, default='EUR')
    # Surface in square meters
    livingSpace = FloatCol(default=None)
    # Price per square meter
    pricePerSqm = FloatCol(default=None)
    # True if the flat or house is furnished
    furnished = BoolCol(default=None)
    # y if realtor, n if rented by a physical person
    realtor = BoolCol(default=None)
    # The name of the realtor or person offering the flat
    realtorName = UnicodeCol(length=300, dbEncoding='utf8', default=None)
    # Latitude
    latitude = FloatCol(default=None)
    # Longitude
    longitude = FloatCol(default=None)
    # "y" if there is a balcony/terrasse
    balcony = BoolCol(default=None)
    # The year the building was built
    yearConstructed = StringCol(length=100, dbEncoding='utf8', default=None)
    # "y" if the flat comes with a cellar
    cellar = BoolCol(default=None)
    # "y" if the flat comes with a parking or a garage
    parking = BoolCol(default=None)
    # House Number in the street
    houseNumber = StringCol(length=100, dbEncoding='utf8', default=None)
    # Street name (incl. "street")
    street = StringCol(length=1000, dbEncoding='utf8', default=None)
    # zip code
    zipCode = StringCol(length=100, default=None)
    # City
    city = UnicodeCol(length=100, dbEncoding='utf8', default=None)
    # y if a lift is present
    lift = BoolCol(default=None)
    # Type of flat (no typology)
    typeOfFlat = StringCol(length=100, default=None)
    # Number of rooms
    noRooms = StringCol(length=10, default=None)
    # Floor the flat is at
    floor = StringCol(length=100, default=None)
    # y if there is a garden
    garden = BoolCol(default=None)
    # y if the flat is wheelchair accessible
    barrierFree = BoolCol(default=None)
    # Country, 2 letter code
    country = StringCol(length=2, default=None)
    # URL of the page
    sourceUrl = StringCol(length=1000, default=None)
    # URL of the page
    description = StringCol(default=None)
    # An ad must be unique on the website
    adIndex = DatabaseIndex('siteId', 'site', unique=True)

class Rent(SQLObject):
    # Total cost
    totalRent = FloatCol(default=None)
    # Surface in square meters
    livingSpace = FloatCol(default=None)
    # Currency, 3 letter code
    currency = StringCol(length=3, default='EUR')
    # Date of arrival
    moveIn = DateCol(default=None)
    # IP of the user hashed
    ipHash = StringCol(length=512,default=None)
    # Date of creating
    createdAt = DateTimeCol(default=DateTimeCol.now)

class Report(SQLObject):
    createdAt = DateTimeCol(default=DateTimeCol.now)                            # Date the ad was first scraped
    country = StringCol(length=2, default=None)                                 # Country, 2 letter code
    site = StringCol(length=30, default=None)                                   # Name of the website
    siteId = StringCol(length=100, default=None)                                # The unique ID from the site where it's scrapped from
    name = StringCol(length=100, default=None)                                  # Name of the report: duplicate, bogus, rent-missing, space-missing, timeout
